@@ -1,7 +1,9 @@
+import inspect
+import io
 import os
 import optparse
-import datetime
-import inspect
+from RestrictedPython import compile_restricted, safe_globals
+import sys
 
 import flask
 from flask import Flask, request, jsonify
@@ -13,8 +15,9 @@ import nest.topology as topo
 from werkzeug.exceptions import abort
 from werkzeug.wrappers import Response
 
-from .api import initializer as api_init
+from .api.initializer import get_arguments
 from .api.client import api_client
+from .exec.helpers import clean_code, get_modules
 from . import scripts
 
 from . import __version__
@@ -53,53 +56,91 @@ def index():
   return jsonify(response)
 
 
+# -------------------------------
+# Execute Python script (caution)
+# -------------------------------
+
+class Capturing(list):
+  """ Monitor stdout contents i.e. print.
+  """
+
+  def __enter__(self):
+    self._stdout = sys.stdout
+    sys.stdout = self._stringio = io.StringIO()
+    return self
+
+  def __exit__(self, *args, **kwargs):
+    self.extend(self._stringio.getvalue().splitlines())
+    del self._stringio    # free up some memory
+    sys.stdout = self._stdout
+
+@app.route('/exec', methods=['GET', 'POST'])
+@cross_origin()
+def route_exec():
+  """ Route to execute script in Python.
+  """
+  try:
+    args, kwargs = get_arguments(request)
+    source_code = kwargs.get('source', '')
+    source_cleaned = clean_code(source_code)
+    byte_code = compile_restricted(source_cleaned, '<inline>', 'exec')
+    locals = get_modules(source_code)
+    with Capturing() as stdout:
+      exec(byte_code, safe_globals, locals)
+    response = {
+        'stdout': '\n'.join(stdout),
+    }
+    if 'return' in kwargs:
+      if isinstance(kwargs['return'], list):
+        data = {}
+        for variable in kwargs['return']:
+          data[variable] = locals.get(variable, None)
+      else:
+        data = locals.get(kwargs['return'], None)
+      response['data'] = nest.hl_api.serializable(data)
+    return jsonify(response)
+  except nest.kernel.NESTError as e:
+      abort(Response(getattr(e, 'errormessage'), 400))
+  except Exception as e:
+      abort(Response(str(e), 400))
+
+
 # --------------------------
 # RESTful API
 # --------------------------
 
 @app.route('/api/nest', methods=['GET'])
+@app.route('/api/nest/', methods=['GET'])
 @cross_origin()
 def router_nest():
-  data, args, kwargs = api_init.data_and_args(request)
-  response = api_client(request, nest_calls, data)
-  return jsonify(response)
+  return jsonify(nest_calls)
 
 
 @app.route('/api/nest/<call>', methods=['GET', 'POST'])
 @cross_origin()
 def router_nest_call(call):
-  data, args, kwargs = api_init.data_and_args(request, call)
-  if call in nest_calls:
-    call = getattr(nest, call)
-    response = api_client(request, call, data, *args, **kwargs)
-  else:
-    data['response']['msg'] = 'The request cannot be called in NEST.'
-    data['response']['status'] = 'error'
-    response = data
+  args, kwargs = get_arguments(request)
+  call = getattr(nest, call)
+  response = api_client(call, args, kwargs)
   return jsonify(response)
 
 
 @app.route('/api/topo', methods=['GET'])
+@app.route('/api/topo/', methods=['GET'])
 @app.route('/api/nest_topology', methods=['GET'])
+@app.route('/api/nest_topology/', methods=['GET'])
 @cross_origin()
 def router_topo():
-  data, args, kwargs = api_init.data_and_args(request)
-  response = api_client(request, topo_calls, data)
-  return jsonify(response)
+  return jsonify(topo_calls)
 
 
 @app.route('/api/topo/<call>', methods=['GET', 'POST'])
 @app.route('/api/nest_topology/<call>', methods=['GET', 'POST'])
 @cross_origin()
 def router_topo_call(call):
-  data, args, kwargs = api_init.data_and_args(request, call)
-  if call in topo_calls:
-    call = getattr(topo, call)
-    response = api_client(request, call, data, *args, **kwargs)
-  else:
-    data['response']['msg'] = 'The request cannot be called in NEST Topology.'
-    data['response']['status'] = 'error'
-    response = data
+  args, kwargs = get_arguments(request)
+  call = getattr(topo, call)
+  response = api_client(call, args, kwargs)
   return jsonify(response)
 
 
@@ -117,9 +158,9 @@ def script(filename, call):
     response = func(request.get_json())
     return jsonify(response)
   except nest.kernel.NESTError as e:
-    abort(Response(getattr(e, 'errormessage').split(':')[-1], 500))
+    abort(Response(getattr(e, 'errormessage').split(':')[-1], 400))
   except Exception as e:
-    abort(Response(str(e), 500))
+    abort(Response(str(e), 400))
 
 
 @app.route('/source', methods=['GET'])
@@ -132,7 +173,7 @@ def inspect_files():
     }
     return jsonify(response)
   except Exception as e:
-    abort(Response(str(e), 500))
+    abort(Response(str(e), 400))
 
 
 @app.route('/source/<filename>', methods=['GET'])
@@ -146,7 +187,7 @@ def inspect_script(filename):
     }
     return jsonify(response)
   except Exception as e:
-    abort(Response(str(e), 500))
+    abort(Response(str(e), 400))
 
 
 @app.route('/source/<filename>/<call>', methods=['GET'])
@@ -161,7 +202,7 @@ def inspect_func(filename, call):
     }
     return jsonify(response)
   except Exception as e:
-    abort(Response(str(e), 500))
+    abort(Response(str(e), 400))
 
 
 if __name__ == "__main__":
